#!/usr/bin/php -q
<?php
# $Header: /home/cvs/cvsroot/cacti/add_device.php,v 1.3 2005/05/20 17:50:03 cvs Exp $

/* do NOT run this script through a web browser */
if (!isset($_SERVER["argv"][0])) {
	die("<br><strong>This script is only meant to run at the command line.</strong>");
}

$no_http_headers = true;

include(dirname(__FILE__)."/include/config.php");
include_once($config["base_path"]."/lib/utility.php");
include_once($config["base_path"]."/lib/api_data_source.php");
include_once($config["base_path"]."/lib/api_graph.php");
include_once($config["base_path"]."/lib/snmp.php");
include_once($config["base_path"]."/lib/data_query.php");
include_once($config["base_path"]."/lib/api_device.php");

#################################################
#
# Retrieve host templates from Cacti's DB
#
function get_host_templates()
{
	$tmparray = db_fetch_assoc("select id, name from host_template order by id");
	foreach ($tmparray as $template)
		$host_templates[$template["id"]] = $template["name"];
	return $host_templates;
}

#################################################
#
# Display host templates
#
function display_host_templates($host_templates)
{
	echo "Valid host templates:\n";
	foreach ($host_templates as $id => $name) {
		echo "\t$id => $name\n";
	}
	echo "\n";
}

#################################################
#
# Display all known communities in Cacti's DB
#
function display_communities()
{
	echo "Known communities are:\n";
	$communities = db_fetch_assoc("select snmp_community from host group by snmp_community");
	foreach ($communities as $community) {
		echo "\t".$community["snmp_community"]."\n";
	}
	echo "\n";
}

#################################################
#
# Retrieve all known hosts from Cacti's DB
#
function get_hosts()
{
	$hosts = array();
	$tmparray = db_fetch_assoc("select id, description from host order by description");
	foreach ($tmparray as $tmp)
		$hosts[$tmp["description"]] = $tmp["id"];

	return $hosts;
}

#################################################
#
# Retrieve all known hostname/IPs from Cacti's DB
#
function get_addresses()
{
	$addresses = array();
	$tmparray = db_fetch_assoc("select id, hostname from host order by hostname");
	foreach ($tmparray as $tmp)
		$addresses[$tmp["hostname"]] = $tmp["id"];

	return $addresses;
}

#################################################
#
# Well, usage
#
function usage()
{
	echo "Usage:
add_device.php templateid description IP snmp_community snmp_version disable

Where:
	- templateid is a number (read below to get a list of templates)
	- description: the name that will be displayed by Cacti in the graphs
	- IP: self explanatory (can also be a FQDN)
	- snmp_community: community string
	- snmp_version: 1/2
	- disable: 1 to add this host but to disable checks and 0 to enable it

Alternative usages:
add_device.php [--host-templates]

Where:
	--host-templates: returns the valid host templates
	--communities: returns the known community strings
";
}

#####################################################################
#
# Main
#
if ($_SERVER["argc"] == 1) {
	usage();
} elseif ($_SERVER["argc"] == 7) {
	$host_templates = get_host_templates();
	$hosts = get_hosts();
	$addresses = get_addresses();
	#foreach ($hosts as $host)
	#	echo "$host\n";
	#foreach ($addresses as $address)
	#	echo "$address\n";

	$template_id = $_SERVER["argv"][1];
	$description = $_SERVER["argv"][2];
	$ip = $_SERVER["argv"][3];
	$community = $_SERVER["argv"][4];
	$snmp_ver = $_SERVER["argv"][5];
	$disable = $_SERVER["argv"][6];

	$snmp_username = "";
	$snmp_password = "";
	$snmp_port = 161;
	$snmp_timeout = 500;
	if (!isset($host_templates[$template_id])) {
		echo "Unknown template id ($template_id)\n";
		exit(1);
	}
	if (isset($hosts[$description])) {
                db_execute("update host set hostname = '$ip' where id = " . $hosts[$description]);
		echo "This host already exists in the database ($description) device-id: (" . $hosts[$description] . ")\n";
		exit(1);
	}
	if (isset($addresses[$ip])) {
                db_execute("update host set description = '$description' where id = " . $addresses[$ip]);
		echo "This IP already exists in the database ($ip) device-id: (" . $addresses[$ip] . ")\n";
		exit(1);
	}
	if ($snmp_ver != "1" && $snmp_ver != "2") {
		echo "Invalid snmp version ($snmp_ver)\n";
		exit(1);
	}
	if ($disable != "1" && $disable != "0") {
		echo "Invalid disable flag ($disable)\n";
		exit(1);
	}
	if ($disable == "0")
		$disable = "";
	else
		$disable = "on";
	echo "Adding $description ($ip) as \"".$host_templates[$template_id]."\" using SNMP v$snmp_ver with community \"$community\"\n";
	$host_id = api_device_save(0, $template_id, $description, $ip,
                        $community, $snmp_ver, $snmp_username, $snmp_password,
                        $snmp_port, $snmp_timeout, $disable);
	if (is_error_message()) {
		echo "Failed to add this device\n";
		return 1;
	} else {
		echo "Success - new device-id: ($host_id)\n";
	}
} else {
	for ($i = 1; $i < $_SERVER["argc"]; $i++) {
		switch ($_SERVER["argv"][$i]) {
			case "--communities":
				display_communities();
				break;
			case "-h":
			case "--h":
			case "--help":
				usage();
				break;
			case "--host-templates":
				display_host_templates(get_host_templates());
				break;
			default:
				break;
		}
	}
}
return 0;
?>
